@@ -1,15 +1,15 @@
 use crate::iu::IU;
-use crate::RustIUMQTT;
+use crate::pyo3_asyncio;
+use crate::{generate_python_exception_string, RustIUMQTT};
 use ipaacar_core::backend::mqtt::MqttBackend;
 use ipaacar_core::components;
+use log::error;
 use pyo3::exceptions::PyTypeError;
+use pyo3::prelude::PyAnyMethods;
 use pyo3::types::{PyModule, PyType};
 use pyo3::{intern, pyclass, pymethods, Bound, Py, PyAny, PyErr, Python};
 use std::sync::Arc;
-use log::error;
-use pyo3::prelude::PyAnyMethods;
 use tokio::sync::RwLock;
-use crate::pyo3_asyncio;
 
 type RustMqttInputBuffer = components::buffer::input::InputBuffer<MqttBackend>;
 
@@ -159,7 +159,8 @@ impl InputBuffer {
                             .unwrap()
                     });
                     if let Err(e) = cor.await {
-                        error!("executing on_new_iu callback for InputBuffer {} failed: {e}", ib_uid)
+                        let err_msg = generate_python_exception_string(e);
+                        error!("executing on_new_iu callback for InputBuffer {ib_uid} failed:\n{err_msg}");
                     }
                 })
             })
@@ -205,8 +206,8 @@ impl InputBuffer {
                 let coroutine = Python::with_gil(|p| {
                     callback
                         .call_method1(p, // calling an async function returns the coroutine
-                                       intern!(p, "process_message_callback"),
-                                       (msg,)
+                                      intern!(p, "process_message_callback"),
+                                      (msg,),
                         ).unwrap()
                 });
                 let ib_uid = ib_uid.clone();
@@ -217,7 +218,8 @@ impl InputBuffer {
                             .unwrap()
                     });
                     if let Err(e) = cor.await {
-                        error!("executing on_new_message callback for InputBuffer {} failed: {e}", ib_uid)
+                        let err_msg = generate_python_exception_string(e);
+                        error!("executing on_new_message callback for InputBuffer {ib_uid} failed:\n{err_msg}");
                     }
                 })
             })
