@@ -1,4 +1,4 @@
-use crate::RustIUMQTT;
+use crate::{generate_python_exception_string, RustIUMQTT};
 use ipaacar_core::components;
 use pyo3::exceptions::{PyNotImplementedError, PyTypeError};
 use pyo3::prelude::{PyAnyMethods, PyModule};
@@ -354,12 +354,12 @@ impl IU {
                     ));
                 }
             };
-            
+
             let locals = Python::with_gil(|p| {
                 // run callback on event loop that awaited add_callback
                 Arc::new(pyo3_asyncio::tokio::get_current_locals(p).unwrap())
             });
-            
+
             let callback = Py::clone(&callback);
             let iu_id = iu.get_uid().await.to_string();
             iu.on_update(move |iu: RustIUMQTT| {
@@ -377,7 +377,8 @@ impl IU {
                             .unwrap()
                     });
                     if let Err(e) = cor.await {
-                        error!("executing on_update callback for IU {} failed: {e}", iu_id)
+                        let err_msg = generate_python_exception_string(e);
+                        error!("executing on_update callback for IU {iu_id} failed:\n{err_msg}");
                     }
                 })
             })
