@@ -97,3 +97,19 @@ fn components(_py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(create_mqtt_pair, m)?)?;
     Ok(())
 }
+
+fn generate_python_exception_string(e: PyErr) -> String {
+    Python::with_gil(|p| {
+        let traceback = e.traceback_bound(p);
+        if traceback.is_none() {
+            return format!("no traceback available.\nException:\n{e}");
+        }
+        let traceback_formatted = traceback.unwrap().format();
+        match traceback_formatted {
+            Ok(t) => { t }
+            Err(te) => {
+                format!("traceback available but formatting failed: {te}.\nException:\n{e}")
+            }
+        }
+    })
+}
